# Copyright 2005 Robert W. Brewer <rwb123 at gmail dot com>
# All Rights Reserved.
#
# This file is part of SpeechLion.
#
# SpeechLion is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# SpeechLion is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SpeechLion; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

from java.awt import AWTKeyStroke
from java.awt.event import KeyEvent
from java.awt.event import InputEvent
from java.awt import Robot
from java.awt import MouseInfo

import string


class InputEventGenerator:
    def __init__(self):
        self.keyCodeDict = self.createKeyCodeDict()
        self.robot = Robot()
        self.robot.setAutoWaitForIdle(0)
        self.robot.setAutoDelay(15)

    def generateKeyEvents(self, keyStr):
        for c in keyStr:
            self.generateKeyEvent(c)
            #self.robot.delay(5)

    ##############################################
    # private stuff

    BUTTONSPEC = {
       'button1':   InputEvent.BUTTON1_MASK,
       'button2':   InputEvent.BUTTON2_MASK,
       'button3':   InputEvent.BUTTON3_MASK
       }
      

    KEYSPEC = {
        ' ':         'SPACE',
        ',':         'COMMA',
        '.':         'PERIOD',
        ':':         'COLON',
        ';':         'SEMICOLON',
        '/':         'SLASH',
        '_':         'UNDERSCORE',
        'tab':       'TAB',
        '\'':        'QUOTE',
#        '"':         'QUOTEDBL',
#        '(':         'LEFT_PARENTHESIS',
#        ')':         'RIGHT_PARENTHESIS',
#        '!':         'EXCLAMATION_MARK',
        '$':         'DOLLAR',
#        '^':         'CIRCUMFLEX',
#        '`':         'BACKQUOTE',
#        '{':         'BRACELEFT',
#        '}':         'BRACERIGHT',
#        '\\':        'BACKSLASH',
#        '&':         'AMPERSAND',
#        '*':         'ASTERISK',
#        '[':         'OPEN_BRACKET',
#        ']':         'CLOSE_BRACKET',
        '@':         'AT',
#        '=':         'EQUALS',
#        '+':         'PLUS',
#        '-':         'MINUS',
#        '#':         'NUMBER_SIGN',
#        '<':         'LESS',
#        '>':         'GREATER',
        'shift':     'SHIFT',
        'ctrl':      'CONTROL',
        'alt':       'ALT',
        'meta':      'META',
        'compose':   'COMPOSE',
        'backspace': 'BACK_SPACE',
        'delete':    'DELETE',
        'insert':    'INSERT',
        'enter':     'ENTER',
        'escape':    'ESCAPE',
        'home':      'HOME',
        'end':       'END',
        'pageup':    'PAGE_UP',
        'pagedown':  'PAGE_DOWN',
        'left':      'LEFT',
        'right':     'RIGHT',
        'up':        'UP',
        'down':      'DOWN',
        'f1':        'F1',
        'f2':        'F2',
        'f3':        'F3',
        'f4':        'F4',
        'f5':        'F5',
        'f6':        'F6',
        'f7':        'F7',
        'f8':        'F8',
        'f9':        'F9',
        'f10':        'F10',
        'f11':        'F11',
        'f12':        'F12'
        }
        
        
    
    def createKeyCodeDict(self):
        """create dictionary mapping characters to a list of keycodes.
        Each keycode must be 'pressed' in order and then 'released'
        in reverse order.
        """
        keyDict = {}

        # lowercase letters
        for c in string.lowercase:
            keyDict[c] = [self.getKeyCode('pressed ' + string.upper(c))]

        # digits
        for c in string.digits:
            keyDict[c] = [self.getKeyCode('pressed ' + c)]

        # uppercase letters
        for c in string.uppercase:
            keyDict[c] = [self.getKeyCode('pressed SHIFT'),
                          self.getKeyCode('pressed ' + c)]

        for c in self.KEYSPEC.keys():
            keyDict[c] = [self.getKeyCode('pressed ' + self.KEYSPEC[c])]

        return keyDict

    def getKeyCode(self, keyCodeStr):
        s = AWTKeyStroke.getAWTKeyStroke(keyCodeStr)
        return s.getKeyCode()
        

    def generateKeyEvent(self, listOrKey):
        # Whatever is input here should resolve
        # to a single "key" event consisting
        # 0 or more modifiers and 1 non-modifier key.
        # valid inputs: 
        #            ['ctrl', 'k']
        #            '-'
        #            'ctrl-k'
        #            ['ctrl-k']
        #            'K'
        #          etc.
        keyCodeList = []
        keyList = []
        if isinstance(listOrKey, type([])):
            keyList += listOrKey
        else:
            keyList = [listOrKey]

        for k in keyList:
            if k == '-':
                keyCodeList += self.keyCodeDict[k][:]
            else:
                ks = k.split('-')
                for j in ks:
                    keyCodeList += self.keyCodeDict[j][:]
            
        for c in keyCodeList:
            #print 'keypress', c
            self.robot.keyPress(c)
            #self.robot.delay(1)

        keyCodeList.reverse()
        for c in keyCodeList:
            #print 'keyrelease', c
            self.robot.keyRelease(c)
            #self.robot.delay(1)
        
    def generateMouseEvent(self, mouseList):
        # mouseList contains a single mouseevent
        # in any order.  The event is made up
        # of a click or press, and a button.
        assert len(mouseList)
        clicks = 0
        press = 0
        release = 0
        x_rel = 0
        y_rel = 0
        wheel = 0
        mouseDist = 15
        button = self.BUTTONSPEC['button1']

        if 'click' in mouseList:
            clicks = 1
        elif 'doubleclick' in mouseList:
            clicks = 2
        elif 'tripleclick' in mouseList:
            clicks = 3
        elif 'press' in mouseList:
            press = 1
        elif 'release' in mouseList:
            release = 1
        elif 'north' in mouseList:
            y_rel = -mouseDist
        elif 'northeast' in mouseList:
            y_rel = -mouseDist
            x_rel = mouseDist
        elif 'east' in mouseList:
            x_rel = mouseDist
        elif 'southeast' in mouseList:
            y_rel = mouseDist
            x_rel = mouseDist
        elif 'south' in mouseList:
            y_rel = mouseDist
        elif 'southwest' in mouseList:
            y_rel = mouseDist
            x_rel = -mouseDist
        elif 'west' in mouseList:
            x_rel = -mouseDist
        elif 'northwest' in mouseList:
            y_rel = -mouseDist
            x_rel = -mouseDist
        elif 'wheelup' in mouseList:
            wheel = -1
        elif 'wheeldown' in mouseList:
            wheel = 1
        else:
            assert 0

        for b in mouseList:
            if self.BUTTONSPEC.has_key(b):
                button = self.BUTTONSPEC[b]
                break

        if press:
            self.robot.mousePress(button)

        if release:
            self.robot.mouseRelease(button)

        for i in range(clicks):
            self.robot.mousePress(button)
            self.robot.mouseRelease(button)
                
        if x_rel or y_rel:
            # since we use the current pointer location
            # to decide where to go next, repeated mouse
            # movements weren't going far enough since the
            # pointer location hadn't caught up to the events.
            # So we wait for the event queue to idle before
            # checking the current location.  Another
            # way to handle this problem would be to pass
            # the repeat count into here, and this would
            # check the current location before moving
            # the mouse.
            self.robot.waitForIdle()
            point = MouseInfo.getPointerInfo().getLocation()
            newx = x_rel + round(point.getX())
            newy = y_rel + round(point.getY())
            self.robot.mouseMove(int(newx), int(newy))

        if wheel:
            self.robot.mouseWheel(wheel)


if __name__ == '__main__':
    k = InputEventGenerator()
    k.generateKeyEvents('Hello There1 @:,$')
    k.generateKeyEvent(['shift', '2'])
