# Copyright 2005 Robert W. Brewer <rwb123 at gmail dot com>
# All Rights Reserved.
# 
# This file is part of SpeechLion.
#
# SpeechLion is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# SpeechLion is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with SpeechLion; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

from edu.cmu.sphinx.frontend.util import *
from edu.cmu.sphinx.recognizer import *
from edu.cmu.sphinx.result import *
from edu.cmu.sphinx.util.props import *
from edu.cmu.sphinx.util.props import *

from java.net import URL

import os
import sys

import InputEventGenerator

import TTS


class Recognizer:
    def __init__(self, sphinxRecognizer, configManager):
        self.recognizer = sphinxRecognizer
        self.configManager = configManager
        self.recognizer.allocate()
        self.inputEventGenerator = InputEventGenerator.InputEventGenerator()
        self.repeat = 1
        self.tags = []
        self.acknowledge = 1

        try:
            self.tts = TTS.TTS()
        except RuntimeError:
            print 'FreeTTS not found, spoken acknowledgements disabled'
            self.tts = None

    def __del__(self):
        self.recognizer.deallocate()


    def say(self, text):
        # this is here to allow easy stubbing out of
        # TTS by simply commenting out the import statement
        # and its constructor
        if self.tts and self.acknowledge:
            self.tts.say(text)

    def printGrammarStrings(self):
        """gets a bunch of random sentences from the
        current grammar, eliminates duplicates, and
        prints them in sorted order.
        """
        grammar = self.configManager.lookup('jsgfGrammar')
        # FIXME wanted to use a dict here, but for some
        # reason was having problems iterating over
        # it using 'for s in sDict' and things like that.
        # A list is slower for finding duplicates, but
        # efficiency is no big deal here and it works.
        sList = []
        for i in range(50):
            s = grammar.getRandomSentence()
            if s not in sList:
                sList.append(s)
        sList.sort()
        for s in sList:
            print s
            
        
    def recognize(self):
        result = self.recognizer.recognize()
        if result:
            s = result.getBestFinalResultNoFiller()
            print 'you said', s
            
            if s == '<unk>':
                self.say('please repeat')
            else:
                self.say(s)
                
            self.execute(s)
        else:
            print 'couldn\'t understand'
            self.say('please repeat')
        return s

    def execute(self, resultStr):
        newtags = self.getTagsList(resultStr)
        print 'tags:-' + '-'.join(newtags) + '-'
        for t in newtags:
            t = t.strip()
            if len(t):
                self.tags.append(t)
                self.execute_tags()
 

    def execute_tags(self):
        assert len(self.tags)
        lastParse = self.tags[-1].split('=')
        if lastParse[0] == 'keymod' or lastParse[0] == 'mousemod':
            # keymod and mousemod tags must stack up until a 
            # key tag or mouse tag is received.
            # all other tags are executed immediately.
            return
        valList = []
        tagType = ''
        for t in self.tags:
            parse = t.split('=')
            assert len(parse) == 1 or len(parse) == 2
            if len(parse) == 2:
                assert len(parse[1]) >= 3
                assert parse[1][0] == '"'
                assert parse[1][-1] == '"'
            if tagType:
                if tagType == 'keymod':
                    assert parse[0] == tagType or parse[0] == 'key'
                elif tagType == 'mousemod':
                    assert parse[0] == tagType or parse[0] == 'mouse'
            else:
                tagType = parse[0]
            if len(parse) == 2:
                valList.append(parse[1][1:-1])
        for i in range(self.repeat):
            if tagType == 'mouse' or tagType == 'mousemod':
                self.inputEventGenerator.generateMouseEvent(valList)
            elif tagType == 'key' or tagType == 'keymod':
                self.inputEventGenerator.generateKeyEvent(valList)
            elif tagType == 'grammar':
                assert len(valList) == 1
                grammar = self.configManager.lookup('jsgfGrammar')
                grammar.loadJSGF(valList[0])
            elif tagType == 'keys':
                assert len(valList) == 1
                self.inputEventGenerator.generateKeyEvents(valList[0])
            elif tagType == 'repeat':
                assert len(valList) == 1
                self.repeat = int(valList[0])
            elif tagType == 'shell':
                assert len(valList) == 1
                os.system(valList[0])
            elif tagType == 'acknowledge':
                assert len(valList) == 1
                self.setAcknowledge(valList[0])
            elif tagType == 'help':
                assert not len(valList)
                self.printGrammarStrings()
            else:
                assert 0
        if tagType != 'repeat':
            self.repeat = 1
        self.tags = []
            

    def setAcknowledge(self, mode):
        if mode == 'off':
            self.acknowledge = 0
        elif mode == 'on':
            # nice touch to say "acknowledge on"
            # when switching into ack mode
            # we put it into an if test
            # so it is not spoken twice when
            # ack mode is already on.
            if not self.acknowledge:
                self.acknowledge = 1
                self.say('acknowledge on')
        else:
            assert 0
        

    def getTagsList(self, resultStr):
        tags = []
        grammar = self.configManager.lookup('jsgfGrammar')
        ruleGrammar = grammar.getRuleGrammar()
        ruleParse = ruleGrammar.parse(resultStr, None)
        if ruleParse:
            tags = ruleParse.getTags()
        return tags
    


def main():
    print 'SpeechLion', open('version.txt').read() 
    url = 'file://' + os.path.join(os.getcwd(), 'speechlion.config.xml')
    url = URL(url)
    
    cm = ConfigurationManager(url)

    recognizer = Recognizer(cm.lookup('recognizer'), cm)
    microphone = cm.lookup('microphone')

    
    if microphone.startRecording():
        print 'recording'
        while 1:
            print 'start speaking'
            s = recognizer.recognize()
    else:
        print 'error starting recording'
    


if __name__ == '__main__':
    main()
    
